package jobshop;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.constraints.Constraint;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.solver.variables.SetVar;
import org.chocosolver.solver.*;
import org.chocosolver.solver.exception.SolverException;
import org.chocosolver.solver.variables.BoolVar;
import org.chocosolver.util.ESat;
import java.io.*;
import java.util.Date;
import org.chocosolver.solver.search.strategy.Search;
import static org.chocosolver.solver.search.strategy.assignments.DecisionOperatorFactory.makeIntSplit;
import org.chocosolver.solver.search.strategy.selectors.values.IntDomainMiddle;
import org.chocosolver.solver.search.strategy.selectors.values.IntDomainMin;
import org.chocosolver.solver.search.strategy.selectors.variables.FirstFail;
import org.chocosolver.solver.search.strategy.strategy.IntStrategy;

public class Jobshop {
 public static void main(String[] args) {
  
  // les données du problème
  // -----------------------
  
  int n=-1;   // 3 jobs
  int m=-1;   // 3 machines
  int Dmax = 970; // horizon de taille 100        
  int[][] M = null;
  int[][] P = null;   
  int[][] Tab = null;   
  int[][] Position = null;
  
  int[] Somme_par_job =null;
  // lecture du fichier
  

        try{
        
            File  source;
           // source = new File("C:\\Users\\lacomme\\Cloud_William\\partage_matthieu\\PPC\\document_tuto\\livre\\Code\\Chapitre_2\\jobshop_inst_litterature\\instance\\la10.txt");
            source = new File("./instance/la10.txt");
 
            FileReader Input=new FileReader(source);
            StreamTokenizer   Lecteur;
            Lecteur=new StreamTokenizer (Input);
            Lecteur.parseNumbers();

            int nombre_ligne=-1;

            // nb ligne        
            Lecteur.nextToken();
            if (Lecteur.ttype==Lecteur.TT_NUMBER)
            {    
            nombre_ligne=(int) Lecteur.nval;
          //  System.out.println("Nombre lu = "+nombre_ligne);
            n=nombre_ligne;
            }

            // nb colonne
            int nombre_colonne=-1;
            Lecteur.nextToken();
            if (Lecteur.ttype==Lecteur.TT_NUMBER)
            {    
            nombre_colonne=(int) Lecteur.nval;
         //   System.out.println("Nombre lu = "+nombre_colonne);
            m=nombre_colonne;
            }
            
            // creation
            M = new int[n][m];
            P = new int[n][m];   
            Tab = new int[n][m];   
            
            Somme_par_job = new int[n];  
           
            int compteur = 0;
            
            for (int i=0; i<nombre_ligne; i++)
            {
                Somme_par_job[i]=0;
                for (int j=0; j<nombre_colonne; j++)
                {
                    Tab[i][j] = compteur;
                    compteur++;
                    
                    // lecture machine
                    Lecteur.nextToken();
                    if (Lecteur.ttype==Lecteur.TT_NUMBER)
                    {    
                    M[i][j]=1+(int) Lecteur.nval;
                   // System.out.println("Nombre lu = "+M[i][j]);
                    }
                    // lecture processing
                    Lecteur.nextToken();
                    if (Lecteur.ttype==Lecteur.TT_NUMBER)
                    {    
                    P[i][j]=(int) Lecteur.nval;
                    
                    
                     Somme_par_job[i]= Somme_par_job[i]+P[i][j];
                    
                 //   System.out.println("Nombre lu = "+P[i][j]);
                    }
                }
            }

        }catch (Exception e) {
                          System.out.println(e.getMessage());
                          };    
  
  
  // fin lecture
  
  Model mon_modele = new Model("JobShop");       
  
  
  
  // 2. variables
  IntVar[] date_debut = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
     int pos = Tab[i][j];
     date_debut[pos] = mon_modele.intVar("ES_"+i+"_"+j, 0, Dmax);
     }
  }

  IntVar[] durees = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
     int pos = Tab[i][j];    
     durees[pos] = mon_modele.intVar("p_"+i+"_"+j, P[i][j], P[i][j]);
     }
  }
  
  IntVar[] date_fin = new IntVar[n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
        int pos = Tab[i][j]; 
        date_fin[pos] = mon_modele.intVar("EF_"+i+"_"+j, 0, Dmax);
     }
  } 

  BoolVar[][] b = new BoolVar [n*m][n*m];
  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
    for (int u=0; u<n; u++){
       for (int v=0; v<m; v++){
        int pos1 = Tab[i][j]; 
        int pos2 = Tab[u][v]; 
        b[pos1][pos2] = mon_modele.boolVar("b_"+i+"_"+j+"_"+u+"_"+v);        
        
       }
    }
   }
  }  
  
  IntVar OBJ = mon_modele.intVar("Cmax", 0, 999);
  
  // contrainte 1. relation entre date de debut et de fin
  int[] coeffs = new int[3];
  coeffs[0] = 1;  // EF
  coeffs[1] = -1;  //ES
  coeffs[2] = -1; //P

  for (int i=0; i<n; i++){
     for (int j=0; j<m; j++){
     int cour = Tab[i][j];
     
       
     IntVar[] colonne = new IntVar[3]; 
     colonne[0] = date_fin[cour];
     colonne[1] = date_debut[cour];
     colonne[2] = durees[cour];
     
     mon_modele.scalar(colonne, coeffs,"=",0).post();
     }
  }  
  
  // contrainte 2 : gamme
  for (int i=0; i<n; i++){
     for (int j=1; j<m; j++){
     int prec =Tab[i][j-1];
     int cour = Tab[i][j];;
     mon_modele.arithm(date_fin[prec], "<=",date_debut[cour]).post();
     }
  }  
    
  // contrainte 3 : disjonction 
  for (int i=0; i<n-1; i++)
  {  
   
     for (int j=0; j<m; j++)
     {
     int ope_i = Tab[i][j];
     for (int u=i+1; u<n; u++)
    {
     for (int v=0; v<m; v++)
     {
        //System.out.println("i= " + i + " j= "+j+" u= " + u + " v= "+v); 
     //   if ( (i==0) && (j==0) && (u==2) && (v==1) )
    //   System.out.println("ah");
        int ope_u = Tab[u][v];    
        if (M[i][j]==M[u][v])
        {
        // System.out.println("ope_i= " + ope_i + " ope_u= "+ope_u);

         mon_modele.ifThenElse(
        b[ope_i][ope_u]  ,
        mon_modele.arithm(date_fin[ope_i],"<=",date_debut[ope_u]),
        mon_modele.arithm(date_fin[ope_u],"<=",date_debut[ope_i])
        );
        }
     }
    } 
     }
  } 
 
   
  // fonction objectif
  
  for (int i = 0; i < n; i++) 
  {
    int valeur = i*m+m-1;
    System.out.println("valeur : "+ valeur);
    mon_modele.arithm(date_fin[i*m+m-1], "<=",OBJ).post();
  }

  System.out.println("....");
  mon_modele.arithm(OBJ, "<=",Dmax).post();
  //mon_modele.arithm(OBJ, ">=",958).post();
  

  mon_modele.setObjective(Model.MINIMIZE, OBJ); 
  Solver mon_solveur = mon_modele.getSolver();
  Solution s = new Solution(mon_modele);
      
        mon_solveur.setLDS(n*m);

      // mon_solveur.setSearch(Search.intVarSearch(new FirstFail(mon_modele), 
               //  new IntDomainMiddle(true), makeIntSplit(), date_debut));

        // contraintes sur les intervalles : efficacite 0
        T_Consommation propage  = new  T_Consommation
                                      (date_debut,
                                       durees,
                                       date_fin, 
                                       b, 
                                       OBJ,
                                       n,m,
                                       Tab,
                                       M);
        
        Constraint contrainte_Intervalle = new Constraint("Contrainte Intervalle", propage);
        contrainte_Intervalle.post(); 
                                                                                 
        
        
      
      Date heure_debut= new Date();
      long h_debut = heure_debut.getTime();
      while (mon_solveur.solve()) {
        s.record();
        int valeur = (int) mon_solveur.getBestSolutionValue();
        Date heure_fin = new Date();
        long h_fin= heure_fin.getTime();
        long duree = h_fin-h_debut;
        long duree_s = duree/1000;
        System.out.println("temps : " + duree_s + " s" + "    Cmax = " + valeur);
        }  
  

      if (mon_solveur.isFeasible() == ESat.TRUE)
        {
        int cout = s.getIntVal(OBJ);
        System.out.println("Cmax = " + cout);

        for (int i=0; i<n; i++)
        {
          for(int j=0; j<m; j++)
          {
          int position = Tab[i][j];
          int deb = s.getIntVal(date_debut[position]);
          int fin = s.getIntVal(date_fin[position]);
          System.out.print(deb + "/" + fin);
         System.out.print("    ");
         }
         System.out.println();
         
       }
      }
      else
        System.out.println("pas de sol.faisable");
  
  

  
  
 }
}