
package jobshop;

import org.chocosolver.solver.constraints.Propagator;
import org.chocosolver.solver.exception.ContradictionException;
import org.chocosolver.solver.variables.BoolVar;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.util.ESat;

public class T_Consommation  extends Propagator<IntVar>{
    
    int nb_appel_au_propagateur;
    int nb_coupe;
    int nb_fails;
    IntVar[] _date_debut;
    IntVar[] _date_fin;
    IntVar _Obj;   
    IntVar[] _durees;
    BoolVar[][] _b;
    int _n;
    int _m;
    int[][] _Tab;
    int[][] _M;
    
    public T_Consommation(IntVar[] date_debut,
                          IntVar[] durees, 
                          IntVar[] date_fin,
                          BoolVar[][] b, 
                          IntVar Obj,
                          int n, int m,
                          int[][] Tab,
                          int[][] M) 
    {
     super(date_debut);
        
    _date_debut = date_debut;
    _date_fin = date_fin;
    _durees = durees;
    _Obj = Obj;
    _b=b;
    _n = n;
    _m = m;
    _Tab = Tab;
    _M = M;
    nb_appel_au_propagateur=0;
    nb_coupe = 0;
    nb_fails=0;
    }

    
    
    @Override
    public void propagate(int evtmask) throws ContradictionException
    {
      nb_appel_au_propagateur++;

        for (int i=0; i<_n-1; i++)  // pour x, y et z
        {
          for (int j=0; j<_m; j++)  // pour x, y et z
          {
            int position = _Tab[i][j];
            int date_debut_au_plus_tot = _date_debut[position].getLB();
            int date_fin_au_plus_tot = 
               date_debut_au_plus_tot + _durees[position].getValue();

            int date_fin_au_plus_tard = _date_fin[position].getUB();
            int date_debut_au_plus_tard = 
               date_fin_au_plus_tard - _durees[position].getValue();


            if (date_fin_au_plus_tot>date_debut_au_plus_tard)
              { 
               int machine_concernee = _M[i][j];

               for (int i2=i+1; i2<_n; i2++)  // pour x, y et z
               {
                 for (int j2=0; j2<_m; j2++)  // pour x, y et z
                 {
                   if (_M[i2][j2]==machine_concernee)
                   {
                    int position2 = _Tab[i2][j2];
                     int date_debut_au_plus_tot2 =
                      _date_debut[position2].getLB();
                     int date_fin_au_plus_tot2 = 
                       date_debut_au_plus_tot2 +
                         _durees[position2].getValue();

                     int date_fin_au_plus_tard2 =
                      _date_fin[position2].getUB();
                     int date_debut_au_plus_tard2 = 
                     date_fin_au_plus_tard2- _durees[position2].getValue();

                     if (date_fin_au_plus_tot2>date_debut_au_plus_tard2)
                     {                    
                        if ( (date_debut_au_plus_tard2>date_debut_au_plus_tard) 
                             &&
                             (date_debut_au_plus_tard2<date_fin_au_plus_tot) )
                         {
                         nb_coupe++; fails();
                         }
                         if ((date_debut_au_plus_tard>date_debut_au_plus_tard2) 
                            && (date_debut_au_plus_tard<date_fin_au_plus_tot2) )
                          {
                            nb_coupe++; fails();
                           }

                        if ((date_fin_au_plus_tot2>date_debut_au_plus_tard) 
                            && (date_fin_au_plus_tot2<date_fin_au_plus_tot) )
                           {
                            nb_coupe++; fails();
                            }

                        if ((date_fin_au_plus_tot>date_debut_au_plus_tard2) 
                            && (date_fin_au_plus_tot<date_fin_au_plus_tot2) )
                           {
                                nb_coupe++; fails();
                            }
                       } // if
                   } // if
              }
            } 
         } 
       } 
    }
   }
        
   @Override
   public ESat isEntailed() {
        for (IntVar var : vars)  
        {
          if (var.getDomainSize()==0) 
          {
           return ESat.FALSE;
          }
          if (var.getDomainSize()>1)  
          {
            return ESat.UNDEFINED;
          }
            
        }
    return ESat.TRUE;
    }
    
    
    
    }
    

